@@ -1,18 +1,30 @@
 package Crypt::MagicSignatures::Key;
 use strict;
 use warnings;
+use Scalar::Util qw/blessed/;
 use Carp 'carp';
 
 use v5.10.1;
 
 our @CARP_NOT;
+our $VERSION = '0.15_1';
 
-our $VERSION = '0.08';
+use overload '""' => sub { $_[0]->to_string }, fallback => 1;
+
+our $DEFAULT_KEY_SIZE = 512;
 
 # Maximum number of tests for random prime generation = 100
-# Range of valid key sizes = 512 - 2048
+# Range of valid key sizes = 512 - 4096
 # Maximum number length for i2osp and os2ip = 30000
 
+# This implementation uses a blessed array for speed.
+# The array order is [n, e, d, size, emLen].
+
+# TODO: Check extreme values for d, e, n, sign, verify
+# TODO: Improve tests for _bitsize, b64url_encode, b64url_decode,
+#       hex2b64url, b64url2hex
+
+
 use Digest::SHA qw/sha256 sha256_hex/;
 use MIME::Base64 qw(decode_base64 encode_base64);
 
@@ -23,14 +35,14 @@ use Math::BigInt try => 'GMP,Pari';
 use Exporter 'import';
 our @EXPORT_OK = qw(b64url_encode b64url_decode);
 
-# Primitives for Math::Prime::Util
+# Primitive for Math::Prime::Util
 sub random_nbit_prime;
 
 our $GENERATOR;
 
 # Load Math::Prime::Util and Math::Random::Secure
 BEGIN {
-  if (eval "use Math::Prime::Util 0.21 qw/random_nbit_prime/; 1;") {
+  if (eval q{use Math::Prime::Util 0.21 'random_nbit_prime'; 1;}) {
     our $GENERATOR = 1;
   };
 };
@@ -41,17 +53,31 @@ sub new {
   my $class = shift;
   my $self;
 
-  # Is already a MagicKey object
-  return $_[0] if ref $_[0] && ref $_[0] eq __PACKAGE__;
+  # Message to pass to carp on parameter failure
+  state $INVALID_MSG = 'Invalid parameters for MagicKey construction';
+
+  # Check if the passed argument is an object
+  if (my $bl = blessed($_[0])) {
+
+    # It's already a MagicKey object - fine!
+    return $_[0] if $bl eq __PACKAGE__;
+  }
+
+  # Do not support references
+  if (ref $_[0]) {
+
+    # Passed object or reference is invalid
+    carp $INVALID_MSG;
+    return;
+  };
 
   # MagicKey in string notation
-  if (@_ == 1 && index($_[0], 'RSA') >= 0) {
+  if (@_ == 1 && index($_[0], 'RSA.') >= 0) {
 
     my $string = shift;
-    return unless $string;
 
     # New object from parent class
-    $self = bless {}, $class;
+    $self = bless [], $class;
 
     # Delete whitespace
     $string =~ tr{\t-\x0d }{}d;
@@ -62,173 +88,203 @@ sub new {
     # Split MagicKey
     my ($type, $mod, $exp, $private_exp) = split(/\./, $string);
 
-    # The key is incorrect
-    if ($type ne 'RSA') {
-      carp "MagicKey type '$type' is not supported";
+    # RSA.modulus(n).exponent(e).private_exponent(d)?
+
+    # MagicKey modulus is missing or invalid
+    unless ($mod = _b64url_to_hex( $mod )) {
+      carp $INVALID_MSG;
       return;
     };
 
-    # RSA.modulus(n).exponent(e).private_exponent(d)?
-    for ($mod, $exp, $private_exp) {
-      next unless $_;
-      $_ = _b64url_to_hex($_);
-    };
+    $exp = _b64url_to_hex( $exp ) if $exp;
+    $private_exp = _b64url_to_hex( $private_exp ) if $private_exp;
 
     # Set modulus
     $self->n( $mod );
 
     # Set exponent
-    $self->e( $exp );
+    $self->e( $exp ) if $exp;
 
     # Set private key
     $self->d( $private_exp ) if $private_exp;
   }
 
   # MagicKey defined by parameters
-  else {
+  elsif (@_ % 2 == 0) {
     my %param = @_;
 
     # RSA complete description
     if (defined $param{n}) {
 
-      $self = bless {}, $class;
+      $self = bless [], $class;
 
       # Set attributes
       foreach (qw/n e d/) {
-	$self->$_($param{$_}) if exists $param{$_};
+        $self->$_( $param{$_} ) if exists $param{$_};
       };
 
       # Modulus was not defined
-      carp 'Key is not well defined' and return unless $self->n;
+      unless ($self->n) {
+        carp $INVALID_MSG;
+        return;
+      };
     }
 
     # Generate new key
     else {
 
-      # Generator not installed
-      unless ($GENERATOR) {
-	carp 'No Math::Prime::Util installed' and return;
-      };
-
-      # Define key size
-      my $size = $param{size};
-
-      # Size is given
-      if ($size) {
-
-	# Key size is too short or impractical
-	if ($size < 512 || $size > 2048 || $size % 2) {
-	  carp "Key size $size is invalid" and return;
-	};
-      }
-
-      # Default size
-      else {
-	$size = 512;
-      };
-
-      # Public exponent
-      my $e = $param{e};
-
-      # Partial size
-      my $psize = int( $size / 2 );
-
-      my $n;
-      my $m = 100; # Maximum number of rounds
-
-      my ($p, $q);
-
-      # Start calculation of combining primes
-    CALC_KEY:
-
-      # Run as long as allowed
-      while ($m > 0) {
-
-	# Fetch random primes p and q
-	# Uses Bytes::Random::Secure by default
-	$p = random_nbit_prime($psize);
-	$q = random_nbit_prime($psize);
-
-	# Fetch a new prime if both are equal
-	while ($p == $q) {
-	  $q = random_nbit_prime($psize);
-	  unless (--$m > 0) {
-	    $p = $q = Math::BigInt->bzero;
-	    last;
-	  };
-	};
-
-	# Calculate modulus
-	$n = $p * $q;
-
-	# Bitsize is correct based on given size
-	last if _bitsize($n) == $size;
-
-	$m--;
-      };
-
-      unless ($m > 0) {
-	carp 'Maximum rounds for key generation is reached' and return;
-      };
-
-      # Bless object
-      $self = bless {}, $class;
-
-      # Set e
-      $self->e($e) if $e;
-
-      # Calculate phi
-      my $phi = ($p - 1) * ($q - 1);
-
-      # Calculate multiplicative inverse of e modulo phi
-      my $d = $self->e->copy->bmodinv($phi);
-
-      # $d is too short
-      goto CALC_KEY if _bitsize($d) < $size / 4;
-
-      # Store d
-      $self->d($d);
-
-      # Store n
-      $self->n($n);
+      carp 'Key generation in new() is DEPRECATED '
+        . 'in favor of generate()';
+      return $class->generate(@_);
     };
+  }
+
+  # Invalid request
+  else {
+    carp $INVALID_MSG;
+    return;
   };
 
-  # Set size (bitsize length of modulus)
-  $self->{size} = _bitsize( $self->n );
+  # Get size (bitsize length of modulus)
+  my $size = $self->size;
 
   # Size is to small
-  if ($self->{size} < 512 || $self->{size} > 2048)  {
-    carp 'Keysize is out of range' and return;
+  if ($size < 512 || $size > 4096)  {
+    carp 'Keysize is out of range';
+    return;
   };
 
   # Set emLen (octet length of modulus)
-  $self->{emLen} = _octet_len( $self->n );
+  $self->_emLen( _octet_len( $self->n ) );
 
   return $self;
 };
 
 
+# Generate a new MagicKey
+sub generate {
+
+  # Generator not installed
+  unless ($GENERATOR) {
+    carp 'No Math::Prime::Util installed';
+    return;
+  };
+
+  # Check for passing of 
+  my ($class, %param) = @_;
+
+  # Define key size
+  my $size = $param{size};
+
+  # Size is given
+  if ($size) {
+
+    # Key size is too short or impractical
+    if ($size < 512 || $size > 4096 || $size % 2) {
+      carp "Key size $size is invalid";
+      return;
+    };
+  }
+
+  # Default size
+  else {
+    $size = $DEFAULT_KEY_SIZE;
+  };
+
+  # Public exponent
+  my $e = $param{e};
+
+  # Partial size
+  my $psize = int( $size / 2 );
+
+  my $n;
+  my $m = 100; # Maximum number of rounds
+
+  my ($p, $q);
+
+  # Start calculation of combining primes
+ CALC_KEY:
+
+  # Run as long as allowed
+  while ($m > 0) {
+
+    # Fetch random primes p and q
+    # Uses Bytes::Random::Secure by default
+    $p = random_nbit_prime($psize);
+    $q = random_nbit_prime($psize);
+
+    # Fetch a new prime if both are equal
+    while ($p == $q) {
+      $q = random_nbit_prime($psize);
+      unless (--$m > 0) {
+        $p = $q = Math::BigInt->bzero;
+        last;
+      };
+    };
+
+    # Calculate modulus
+    $n = $p * $q;
+
+    # Bitsize is correct based on given size
+    last if _bitsize($n) == $size;
+
+    $m--;
+  };
+
+  unless ($m > 0) {
+    carp 'Maximum rounds for key generation is reached';
+    return;
+  };
+
+  # Bless object
+  my $self = bless [], $class;
+
+  # Set e
+  $self->e($e) if $e;
+
+  # Calculate phi
+  my $phi = ($p - 1) * ($q - 1);
+
+  # Calculate multiplicative inverse of e modulo phi
+  my $d = $self->e->copy->bmodinv($phi);
+
+  # $d is too short
+  goto CALC_KEY if _bitsize($d) < $size / 4;
+
+  # Store d
+  $self->d($d);
+
+  # Store n
+  $self->n($n);
+
+  # Set emLen (octet length of modulus)
+  $self->_emLen( _octet_len( $self->n ) );
+
+  return $self;
+};
+
+
+
 # Get or set modulus
 sub n {
   my $self = shift;
 
   # Get value
-  unless ($_[0]) {
-    return ($self->{n} //= Math::BigInt->bzero);
-  };
+  return $self->[0] unless $_[0];
 
   # Set value
   my $n = Math::BigInt->new( shift );
 
   # n is not a number
-  carp 'n is not a number' and return if $n->is_nan;
+  if ($n->is_nan) {
+    carp 'n is not a number';
+    return;
+  };
 
   # Delete precalculated emLen and size
-  delete $self->{emLen};
-  delete $self->{size};
+  $#{$self} = 2;
 
-  return $self->{n} = $n;
+  $self->[0] = $n;
 };
 
 
@@ -237,17 +293,18 @@ sub e {
   my $self = shift;
 
   # Get value
-  unless ($_[0]) {
-    return ($self->{e} //= Math::BigInt->new('65537'));
-  };
+  return ($self->[1] //= Math::BigInt->new('65537')) unless $_[0];
 
   # Set value
   my $e = Math::BigInt->new( shift );
 
   # e is not a number
-  carp 'e is not a number' and return if $e->is_nan;
+  if ($e->is_nan) {
+    carp 'e is not a number';
+    return;
+  };
 
-  return $self->{e} = $e;
+  $self->[1] = $e;
 };
 
 
@@ -256,25 +313,25 @@ sub d {
   my $self = shift;
 
   # Get value
-  unless ($_[0]) {
-    return $self->{d} // undef;
-  };
+  return ($self->[2] // undef) unless $_[0];
 
   # Set value
   my $d = Math::BigInt->new( shift );
 
   # d is not a number
-  carp 'd is not a number' and return if $d->is_nan;
+  if ($d->is_nan) {
+    carp 'd is not a number';
+    return;
+  };
 
-  return $self->{d} = $d;
+  $self->[2] = $d;
 };
 
 
 # Get key size
 sub size {
-  my $self = shift;
-  return unless $self->n;
-  return $self->{size} // ($self->{size} = _bitsize($self->n));
+  # return unless $_[0]->n;
+  $_[0]->[3] // ($_[0]->[3] = _bitsize($_[0]->n));
 };
 
 
@@ -283,12 +340,13 @@ sub sign {
   my ($self, $message) = @_;
 
   unless ($self->d) {
-    carp 'You can only sign with a private key' and return;
+    carp 'Unable to sign without a private key';
+    return;
   };
 
-  my $encoded_message = _sign_emsa_pkcs1_v1_5($self, $message);
-
-  return b64url_encode($encoded_message);
+  b64url_encode(
+    _sign_emsa_pkcs1_v1_5($self, $message)
+  );
 };
 
 
@@ -298,17 +356,26 @@ sub verify {
       $message,
       $encoded_message) =  @_;
 
-  # Delete whitespace and padding
-  $encoded_message =~ tr{=\t-\x0d }{}d;
-
+  # Missing parameters
   unless ($encoded_message && $message) {
     carp 'No signature or message given';
     return;
   };
 
-  return unless $self->n;
+  # Delete whitespace and padding
+  $encoded_message =~ tr{=\t-\x0d }{}d;
 
-  return _verify_emsa_pkcs1_v1_5(
+  # Invalid message
+  unless ($encoded_message) {
+    carp 'No signature given';
+    return;
+  };
+
+  # No modulus
+  # return unless $self->n;
+
+  # Verify message
+  _verify_emsa_pkcs1_v1_5(
     $self,
     $message,
     # _b64url_to_hex( $encoded_message )
@@ -321,23 +388,17 @@ sub verify {
 sub to_string {
   my $self = shift;
 
-  my $n = $self->n;
-  return '' unless $n;
+  # return '' unless $n; # Shouldn't be possible
 
-  my $e = $self->e;
+  # Convert modulus and exponent and add to component array
+  my @array = ('RSA', _hex_to_b64url($self->n), _hex_to_b64url($self->e));
 
-  # Convert modulus and exponent
-  $_ = _hex_to_b64url($_) for ($n, $e);
-
-  my @array = ('RSA', $n, $e);
-
-  push(@array, _hex_to_b64url($self->d)) if $_[0] && $self->d;
-
-  my $mkey = join('.', @array);
+  if ($_[0] && $self->d) {
+    push(@array, _hex_to_b64url($self->d));
+  };
 
   # Specification is not clear about $mkey =~ s/=+//g;
-
-  $mkey;
+  join('.', @array);
 };
 
 
@@ -353,8 +414,7 @@ sub b64url_encode {
 
   # Trim padding or not
   $v =~ s/\=+$// unless (defined $_[1] ? $_[1] : 1);
-
-  return $v;
+  $v;
 };
 
 
@@ -372,15 +432,14 @@ sub b64url_decode {
     $v .= chr(61) x (4 - $padding);
   };
 
-  return decode_base64($v);
+  decode_base64($v);
 };
 
 
 # Get octet length of n
 sub _emLen {
-  my $self = shift;
-  return 0 unless $self->n;
-  return $self->{emLen} // ($self->{emLen} = _octet_len( $self->n ));
+  # return 0 unless $_[0]->n;
+  ($_[0]->[4] // ($_[0]->[4] = _octet_len( $_[0]->n )));
 };
 
 
@@ -397,9 +456,7 @@ sub _sign_emsa_pkcs1_v1_5 {
   # encode message (Hash digest is always 'sha-256')
   my $EM = _emsa_encode($M, $k) or return;
 
-  my $m = _os2ip($EM);
-  my $s = _rsasp1($K, $m);
-  _i2osp($s, $k); # S
+  _i2osp(_rsasp1($K, _os2ip($EM)), $k);
 };
 
 
@@ -414,7 +471,10 @@ sub _verify_emsa_pkcs1_v1_5 {
 
   # The length of the signature is not
   # equivalent to the length of the RSA modulus
-  carp 'Invalid signature' and return if length($S) != $k;
+  if (length($S) != $k) {
+    carp 'Invalid signature';
+    return;
+  };
 
   my $s = _os2ip($S);
   my $m = _rsavp1($K, $s) or return;
@@ -432,7 +492,8 @@ sub _rsasp1 {
   my ($K, $m) = @_;
 
   if ($m >= $K->n) {
-    carp 'Message representative out of range' and return;
+    carp 'Message representative out of range';
+    return;
   };
 
   return $m->bmodpow($K->d, $K->n);
@@ -448,7 +509,8 @@ sub _rsavp1 {
 
   # Is signature in range?
   if ($s > $K->n || $s < 0) {
-    carp 'Signature representative out of range' and return;
+    carp 'Signature representative out of range';
+    return;
   };
 
   return $s->bmodpow($K->e, $K->n);
@@ -473,7 +535,8 @@ sub _emsa_encode {
   my $tLen = length( $T );
 
   if ($emLen < $tLen + 11) {
-    carp 'Intended encoded message length too short' and return;
+    carp 'Intended encoded message length too short';
+    return;
   };
 
   return "\x00\x01" . ("\xFF" x ($emLen - $tLen - 3)) . "\x00" . $T;
@@ -484,13 +547,12 @@ sub _emsa_encode {
 sub _os2ip {
   # Based on Crypt::RSA::DataFormat
   # See also Convert::ASN1
-
   my $os = shift;
 
-  my $l = length($os);
+  my $l = length $os;
   return if $l > 30_000;
 
-  my $base = Math::BigInt->new(256);
+  state $base = Math::BigInt->new(256);
   my $result = Math::BigInt->bzero;
   for (0 .. $l - 1) {
     # Maybe optimizable
@@ -509,16 +571,16 @@ sub _i2osp {
 
   my $num = Math::BigInt->new(shift);
 
-  return if $num->is_nan;
-  return if $num->length > 30_000;
+  return if $num->is_nan || $num->length > 30_000;
 
   my $l = shift || 0;
-  my $base = Math::BigInt->new(256);
+  state $base = Math::BigInt->new(256);
 
   my $result = '';
 
   if ($l && $num > ( $base ** $l )) {
-    carp 'i2osp error - Integer is to short' and return;
+    carp 'i2osp error - Integer is to short';
+    return;
   };
 
   do {
@@ -541,14 +603,14 @@ sub _i2osp {
 sub _octet_len {
   return Math::BigInt->new( _bitsize( shift ))
     ->badd(7)
-      ->bdiv(8)
-	->bfloor;
+    ->bdiv(8)
+    ->bfloor;
 };
 
 
 # Returns the bitlength of the integer
 sub _bitsize {
-  my $int = Math::BigInt->new( shift );
+  my $int = Math::BigInt->new( $_[0] );
   return 0 unless $int;
   # Trim leading '0b'
   length( $int->as_bin ) - 2;
@@ -562,9 +624,8 @@ sub _b64url_to_hex {
   #         MagicSignatures/SignatureAlgRsaSha256.pm
 
   # Decode and convert b64url encoded hex number
-  return Math::BigInt->new(
-    '0x' . unpack( 'H*', b64url_decode( shift ) )
-  );
+  my $b64 = b64url_decode( shift ) or return;
+  Math::BigInt->new('0x' . unpack( 'H*', $b64));
 };
 
 
@@ -580,7 +641,7 @@ sub _hex_to_b64url {
   $num = ( ( ( length $num ) % 2 ) > 0 ) ? '0' . $num : $num;
 
   # Encode number using b64url
-  return b64url_encode( pack( 'H*', $num ) );
+  b64url_encode( pack( 'H*', $num ) );
 };
 
 
@@ -614,11 +675,9 @@ Crypt::MagicSignatures::Key - MagicKeys for the Salmon Protocol
 L<Crypt::MagicSignatures::Key> implements MagicKeys as described in the
 L<MagicSignatures Specification|http://salmon-protocol.googlecode.com/svn/trunk/draft-panzer-magicsig-01.html>
 to sign messages of the L<Salmon Protocol|http://www.salmon-protocol.org/>.
-MagicSignatures is a I<"robust mechanism for digitally signing nearly arbitrary messages">.
+MagicSignatures is a "robust mechanism for digitally signing nearly arbitrary messages".
 See L<Crypt::MagicSignatures::Envelope> for using MagicKeys to sign MagicEnvelopes.
 
-B<This module is an early release! There may be significant changes in the future.>
-
 
 =head1 ATTRIBUTES
 
@@ -676,18 +735,22 @@ The MagicKey keysize in bits.
     e => 3
   );
 
-  $mkey = Crypt::MagicSignatures::Key->new(size => 1024);
-
 
 The Constructor accepts MagicKeys in
 L<compact notation|http://salmon-protocol.googlecode.com/svn/trunk/draft-panzer-magicsig-01.html#anchor13>
 or by attributes.
 
-If no C<n> attribute is given and L<Math::Prime::Util>
-is installed, a new key will be generated.
+
+=head2 generate
+
+  my $mkey = Crypt::MagicSignatures::Key->new(size => 1024);
+
+Generate a new key. Requires L<Math::Prime::Util> to be installed.
+
+Accepts the attributes C<size> and C<e>.
 In case no C<size> attribute is given, the default key size
 for generation is 512 bits, which is also the minimum size.
-The maximum size is 2048 bits.
+The maximum size is 4096 bits.
 
 
 =head2 sign
@@ -755,6 +818,7 @@ The function can be exported.
   print b64url_decode('VGhpcyBpcyBhIG1lc3NhZ2U=');
 
 Decodes a base-64 string with URL safe characters.
+Characters not part of the character set are silently ignored.
 The function can be exported.
 
 
@@ -765,7 +829,7 @@ other than Perl v5.10.1 and core modules.
 For key generation L<Math::Prime::Util> v0.21 is necessary.
 
 Either L<Math::BigInt::GMP> (preferred) or L<Math::BigInt::Pari>
-are strongly recommended for speed improvement
+is strongly recommended for speed improvement
 (signing and verification) as well as
 L<Math::Prime::Util::GMP> and L<Math::Random::ISAAC::XS>
 (key generation).
@@ -781,7 +845,7 @@ compatible with other implementations!
 
 L<Crypt::MagicSignatures::Envelope>,
 L<Crypt::RSA::DataFormat>,
-L<Alt::Crypt::RSA::BigInt> (which wasn't available when I started this module),
+L<Alt::Crypt::RSA::BigInt>,
 L<https://github.com/sivy/Salmon>.
 
 
@@ -792,9 +856,9 @@ L<https://github.com/sivy/Salmon>.
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright (C) 2012-2013, L<Nils Diewald|http://nils-diewald.de/>.
+Copyright (C) 2012-2016, L<Nils Diewald|http://nils-diewald.de/>.
 
 This program is free software, you can redistribute it
-and/or modify it under the same terms as Perl.
+and/or modify it under the terms of the Artistic License version 2.0.
 
 =cut
